import asyncio
import aiohttp
from urllib.parse import urljoin, urlparse
from typing import Set, Dict, Any, List
from bs4 import BeautifulSoup
from fake_useragent import UserAgent
from tqdm import tqdm
from .analyzer import SEOAnalyzer
import time

class WebsiteCrawler:
    def __init__(self, base_url: str, max_pages: int = 100, delay: float = 1.0):
        self.base_url = base_url
        self.base_domain = urlparse(base_url).netloc
        self.max_pages = max_pages
        self.delay = delay
        self.visited_urls: Set[str] = set()
        self.urls_to_visit: Set[str] = {base_url}
        self.results: Dict[str, Any] = {}
        self.ua = UserAgent()
        self.analyzer = SEOAnalyzer()
        self.session = None

    async def _get_page_content(self, url: str) -> tuple[str, BeautifulSoup]:
        """Fetch page content asynchronously"""
        if not self.session:
            self.session = aiohttp.ClientSession()

        headers = {'User-Agent': self.ua.random}
        async with self.session.get(url, headers=headers) as response:
            content = await response.text()
            return content, BeautifulSoup(content, 'html.parser')

    def _extract_internal_links(self, soup: BeautifulSoup, url: str) -> Set[str]:
        """Extract internal links from the page"""
        internal_links = set()
        for link in soup.find_all('a'):
            href = link.get('href')
            if not href:
                continue

            full_url = urljoin(url, href)
            if urlparse(full_url).netloc == self.base_domain:
                internal_links.add(full_url)

        return internal_links

    async def crawl(self) -> Dict[str, Any]:
        """Crawl the website and analyze each page"""
        with tqdm(total=self.max_pages, desc="Crawling pages") as pbar:
            while self.urls_to_visit and len(self.visited_urls) < self.max_pages:
                url = self.urls_to_visit.pop()
                if url in self.visited_urls:
                    continue

                try:
                    content, soup = await self._get_page_content(url)
                    self.results[url] = self.analyzer.analyze_page(url)
                    new_links = self._extract_internal_links(soup, url)
                    self.urls_to_visit.update(new_links - self.visited_urls)
                    self.visited_urls.add(url)
                    pbar.update(1)
                    await asyncio.sleep(self.delay)  # Respect crawl delay

                except Exception as e:
                    self.results[url] = {'url': url, 'error': str(e)}
                    self.visited_urls.add(url)

        if self.session:
            await self.session.close()

        return self._generate_summary()

    def _generate_summary(self) -> Dict[str, Any]:
        """Generate a summary of the crawl results"""
        total_pages = len(self.visited_urls)
        pages_with_errors = sum(1 for result in self.results.values() if 'error' in result)
        
        return {
            'base_url': self.base_url,
            'total_pages_crawled': total_pages,
            'pages_with_errors': pages_with_errors,
            'crawl_stats': {
                'successful_pages': total_pages - pages_with_errors,
                'failed_pages': pages_with_errors
            },
            'page_results': self.results
        }